import hashlib
import binascii
from messenger2.protocols.JIM import JIM


def get_hash_from_password(password, salt):
    """
    get hash from data
    :param password: user password
    :param salt: salt
    :return: string hash password
    """
    hash_pass = hashlib.pbkdf2_hmac(
        "sha256", password=bytes(
            password, encoding=JIM.ENCODING), salt=bytes(
            salt, encoding=JIM.ENCODING), iterations=100000)
    return str(binascii.hexlify(hash_pass), encoding=JIM.ENCODING)



from PySide2.QtWidgets import QApplication, QMainWindow
from PySide2.QtGui import QStandardItemModel, QStandardItem, QBrush, QColor
from PySide2.QtCore import Qt, QFile, Slot, QEvent
from PySide2.QtUiTools import QUiLoader
import sys
from messenger2.databases.database import ClientDatabase
from messenger2 import config
from messenger2.client.transport import ClientTransport
from messenger2.client.gui.add_contact import AddWindow
from messenger2.client.gui.alert_window import AlertWindow
from messenger2.client.gui.del_contact import DeleteWindow
from messenger2.client.gui.open_dialog import OpenWindow


class ClientWindow(QMainWindow):

    """
    Main client window
    """

    def __init__(self, database, transport, ui_file, username="user"):
        super(ClientWindow, self).__init__()
        self.ui = None
        self.username = username
        self.transport = transport
        self.active_contact = None
        self.database = database
        self.add_window = None
        self.del_window = None
        self.open_dialog = None
        self.alert = None
        self.setUI(ui_file)

        self.transport.alert_message.connect(self.alert_msg)
        self.transport.new_message.connect(self.new_msg)
        self.transport.update_contact_list.connect(self.show_contact_history)
        self.transport.get_contacts()
        self.get_contact_list()

    def setUI(self, ui_file):
        """function that set up ui files"""
        ui = QFile(ui_file)
        ui.open(QFile.ReadOnly)
        loader = QUiLoader()
        self.ui = loader.load(ui)
        ui.close()

        self.ui.menu_exit.triggered.connect(self.close)

        self.ui.add_contact_btn.clicked.connect(self.open_add_window)
        self.ui.menu_add_contact.triggered.connect(self.open_add_window)

        self.ui.del_contact_btn.clicked.connect(self.open_del_window)
        self.ui.menu_del_contact.triggered.connect(self.open_del_window)

        self.ui.send_btn.clicked.connect(self.send_msg)
        self.ui.clear_msg_btn.clicked.connect(self.clear_msg)

        self.ui.contact_list.doubleClicked.connect(
            lambda: self.show_contact_history())
        self.ui.history_list.setHorizontalScrollBarPolicy(
            Qt.ScrollBarAlwaysOff)
        self.ui.history_list.setWordWrap(True)

        self.ui.installEventFilter(self)

    def open_add_window(self):
        """function that opens add window"""
        self.add_window = AddWindow(
            database=self.database,
            transport=self.transport)
        self.add_window.add_contact.connect(self.add_contact)
        self.add_window.show()
        print("add contact")

        pass

    def open_del_window(self):
        """function that opens del window"""
        if self.active_contact is not None:
            self.del_window = DeleteWindow(
                database=self.database,
                transport=self.transport,
                contact=self.active_contact)
            self.del_window.delete_contact.connect(self.del_contact)
            self.del_window.show()
            print("del contact")
        else:
            self.alert = AlertWindow(info_msg="Выберите контакт для удаления")
            self.alert.show()
        pass

    def open_dialog_window(self, new_contact):
        """function that opens new dialog window"""
        if self.database.check_contact(new_contact):
            self.open_dialog = OpenWindow(
                database=self.database,
                transport=self.transport,
                contact=new_contact,
                is_new=False)
        else:
            self.open_dialog = OpenWindow(
                database=self.database,
                transport=self.transport,
                contact=new_contact,
                is_new=True)

        self.open_dialog.add_contact.connect(self.add_contact)
        self.open_dialog.select_history.connect(self.show_contact_history)
        self.open_dialog.show()

    def send_msg(self):
        """function that sends msg to another client"""
        print("send msg")
        if self.active_contact is not None:
            msg = self.ui.msg_window.toPlainText()
            self.transport.send_message(
                message=msg,
                username=self.username,
                to=self.active_contact)
            self.show_contact_history()
            self.clear_msg()
        else:
            self.clear_msg()
            self.alert = AlertWindow(info_msg="Не выбран диалог")
            self.alert.show()
        pass

    @Slot(str)
    def alert_msg(self, msg):
        """function that show alert messages"""
        print("alert signal")
        self.alert = AlertWindow(info_msg=msg)
        self.alert.show()

    def clear_msg(self):
        """function that clear message that is going to be sent to another client"""
        print("clear_msg")
        self.ui.msg_window.setText("")
        pass

    def show(self) -> None:
        """show gui"""
        self.ui.show()

    @Slot(str)
    def add_contact(self, contact):
        """function that add new contact to user list"""
        try:
            self.transport.add_contact(contact)
        except Exception:
            self.alert = AlertWindow(
                info_msg="Не удалось добавить пользователя")

        finally:
            self.get_contact_list()

    @Slot(str)
    def del_contact(self, contact):
        """function that delete contact from user list"""
        try:
            self.transport.del_contact(contact)
            self.show_contact_history(contact="")
        except Exception:
            self.alert = AlertWindow(
                info_msg="Не удалось удалить пользователя")

        finally:
            self.get_contact_list()

    def get_contact_list(self):
        """function that gets contact list of current user"""
        contact_list = self.database.get_contacts()
        print(contact_list)
        model = QStandardItemModel()
        for contact in contact_list:
            if contact.login != "":
                list_contact = QStandardItem(contact.login)
                list_contact.setEditable(False)
                model.appendRow(list_contact)

        self.ui.contact_list.setModel(model)

    @Slot()
    def show_contact_history(self, contact=None):
        """function that shows message history for every selected contact"""
        if contact is None:
            login = self.ui.contact_list.currentIndex().data()
            print(login)
            self.active_contact = login
        else:
            self.active_contact = contact
        print(self.active_contact)
        history_list = self.database.get_contact_history(
            login=self.active_contact)

        model = QStandardItemModel()

        for history in history_list:
            msg = QStandardItem(
                f"{history.user} -> {history.msg} : {history.when}")

            if history.user == self.active_contact:
                msg.setTextAlignment(Qt.AlignRight)
                msg.setBackground(QBrush(QColor("red")))
            else:
                msg.setTextAlignment(Qt.AlignLeft)
                msg.setBackground(QBrush(QColor("green")))
            msg.setEditable(False)
            model.appendRow(msg)

        self.ui.history_list.setModel(model)

    def new_msg(self, send_from):
        """function that shows new message from contact or new contact"""
        if self.active_contact == send_from:
            self.show_contact_history()
        else:
            self.open_dialog_window(new_contact=send_from)

    def closeEvent(self, event) -> None:
        """close window from button"""
        self.transport.quit()
        QApplication.exit()

    def eventFilter(self, obj, event) -> bool:
        """close window from gui"""
        if obj is self.ui:
            if event.type() == QEvent.Close:
                self.transport.quit()
                event.accept()
                return True

        return False


if __name__ == "__main__":
    app = QApplication()
    test_db = ClientDatabase(engine=config.TEST_DATABASE_ENGINE)
    transport = ClientTransport(
        ip_address=config.USER_ADDRESS,
        port=config.USER_PORT,
        database=test_db,
        username="user")
    transport.setDaemon(True)
    transport.start()
    window = ClientWindow(database=test_db, transport=transport)
    window.show()
    sys.exit(app.exec_())



import argparse

from PySide2.QtWidgets import QApplication
from messenger2.client.gui.welcome import WelcomeWindow

from messenger2 import config
from random import randint
import sys


def get_parameters(address=None, port=None, username=None, password=None):
    """
    get parameters from command line
    if there are not in line then from config file
    :param address: ip address
    :param port: port
    :param username: login
    :param password: password
    :return: parameters
    """

    parser = argparse.ArgumentParser(description="client parser")
    parser.add_argument(
        "-p",
        "--port",
        type=int,
        default=config.USER_PORT,
        help="port name")
    parser.add_argument(
        "-a",
        "--address",
        type=str,
        default=config.USER_ADDRESS,
        help="address name")
    parser.add_argument(
        "-u",
        "--username",
        type=str,
        default=f"Guest_{randint(0, 15)}",
        help="username")
    parser.add_argument("-k", "--key", type=str, help="password")
    args = parser.parse_args()

    if address is None:
        address_param = args.address
    else:
        address_param = address

    if port is None:
        port_param = args.port
    else:
        port_param = port

    if username is None:
        user_param = args.username
    else:
        user_param = username

    if password is None:
        password_param = args.key
    else:
        password_param = password

    return address_param, port_param, user_param, password_param


def start(ip=None, port=None, username=None, password=None):
    """
    open welcome window with parameters
    :param ip: ip address
    :param port: port
    :param username: login
    :param password: password
    :return: None
    """
    ip, port, username, password = get_parameters(ip, port, username, password)
    app = QApplication()
    window = WelcomeWindow(
        ip_address=ip,
        port=port,
        username=username,
        password=password)
    window.show()
    sys.exit(app.exec_())


if __name__ == "__main__":
    start()



from Crypto.Cipher import PKCS1_v1_5, AES
from Crypto.Random import get_random_bytes
from Crypto.PublicKey import RSA
from messenger2.common.security.keys import get_server_public_key, get_public_key


def get_info(pk, data):
    """
    encrypt data with public key
    :param pk: rsa public key
    :param data: data to encrypt
    :return: encrypted data
    """
    session_key = get_random_bytes(16)
    rsa = PKCS1_v1_5.new(key=pk)
    enc_session_key = rsa.encrypt(session_key)

    aes = AES.new(session_key, AES.MODE_EAX)
    enc_text, tag = aes.encrypt_and_digest(data)
    enc_data = b"".join([enc_session_key, aes.nonce, tag, enc_text])

    return enc_data


def encript_data(username_pk, data):
    """
    encrypt data on server side
    :param username_pk: user rsa public key
    :param data: data to encrypt
    :return: encrypted data
    """
    username_pk = RSA.import_key(username_pk)
    return get_info(username_pk, data)


def encript_server_data(data):
    """
    encrypt data on client side
    :param data: data to encrypt
    :return: encrypted data
    """
    server_pk = get_server_public_key()
    return get_info(server_pk, data)


if __name__ == "__main__":
    pass


from PySide2.QtWidgets import QDialog
from PySide2.QtCore import QFile, Signal
from PySide2.QtUiTools import QUiLoader
from messenger2 import config
import os


class DeleteWindow(QDialog):
    """
    Window for deleting user contacts
    """

    delete_contact = Signal(str)
    send_alert = Signal(str, str, str)

    def __init__(self, database, transport, contact):
        super(DeleteWindow, self).__init__()
        self.database = database
        self.transport = transport
        self.contact = contact
        self.alert = None
        self.send_alert.connect(self.transport.send_alert)
        self.setUI(os.path.join(config.CLIENT_UI_DIR, "del_client.ui"))

    def setUI(self, ui_file):
        """function that set up ui files"""
        ui = QFile(ui_file)
        ui.open(QFile.ReadOnly)
        loader = QUiLoader()
        self.ui = loader.load(ui)
        ui.close()

        self.ui.user_label.setText(self.contact)
        self.ui.yes_btn.clicked.connect(self.yes)
        self.ui.no_btn.clicked.connect(self.no)

    def yes(self):
        """function that emits when client clicked yes button"""
        self.delete_contact.emit(self.contact)
        self.send_alert.emit(
            f"Пользователь {self.transport.username} удалил вас из контактов",
            self.contact,
            self.transport.username)
        self.close()

    def no(self):
        """close window if user decided not to delete contact"""
        self.close()

    def close(self) -> bool:
        """close window"""
        return self.ui.close()

    def show(self) -> None:
        """show gui"""
        self.ui.show()



import threading
from PySide2.QtCore import QObject, Signal, Slot
import socket
from messenger2.protocols.JIM import JIM
from messenger2 import config
import time
from messenger2.common.decorators import log_exception
from messenger2.databases.database import ClientDatabase
from messenger2.common.security.keys import get_public_key, get_client_server_public_key, save_server_public_key, generate_pair
from messenger2.common.security.encript_data import encript_server_data
from messenger2.common.security.decript_data import decript_data
from json import JSONDecodeError

LOCK = threading.Lock()


class ClientTransport(threading.Thread, QObject):

    """
    Main Transport between client and server.
    Send all requests and proceed responses
    """

    new_message = Signal(str)
    alert_message = Signal(str)
    update_contact_list = Signal()
    connection_lost = Signal()

    def __init__(self, ip_address, port, username, password):
        QObject.__init__(self)
        super(ClientTransport, self).__init__()

        self.database = None
        self.port = port
        self.address = ip_address
        self.username = username
        self.password = password
        self.socket = None
        self.is_active = False

    def connect_to_server(self):
        """
        try to make a connection with server
        If auth is wrong close socket
        """
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.settimeout(5)
        self.socket.connect((self.address, self.port))

        result = self.join_to_server()
        if result:
            engine = f"{config.CLIENT_DATABASE_ENGINE}{self.username}.db3"
            self.database = ClientDatabase(engine=engine)
            self.is_active = True

    def proceed_answer(self):
        """function that proceed response from server"""
        response = self.get_answer()
        protocol = self.get_protocol_from_response(response)
        msg, send_to, send_from = protocol.get_message_info()
        print("get_answer")
        print(protocol.request.get("action"))

        if protocol.presence_type:
            print("presense")
            server_key = get_client_server_public_key(to_str=True)
            server_public_key = protocol.get_public_key()
            if server_public_key == server_key:
                print("server is equal")
            if server_key is None or server_public_key != server_key:
                print("saving server key")
                save_server_public_key(server_public_key)

        if protocol.response_type:
            print("response")
            if protocol.request.get("response") == 200:
                if msg is not None:
                    print(msg)
                return True
            else:
                return False

        elif protocol.message_type:
            if send_to == self.username:
                self.database.save_msg(user=send_from, to=send_to, msg=msg)
                self.new_message.emit(send_from)

        elif protocol.get_contacts_type:
            print(send_from)
            if send_from is not None:
                print(send_from)
                self.database.add_clients(msg)
            else:
                return msg

        elif protocol.add_type:
            self.database.add_client(login=msg)

        elif protocol.del_type:
            print("delete")
            self.database.del_client(login=msg)
            self.database.delete_contact_history(login=msg)

        elif protocol.alert_type:
            print("alert")
            print(protocol.get_info())
            if protocol.get_info() == "delete_contact_history":
                print(send_from)
                self.database.delete_contact_history(login=send_from)
                self.update_contact_list.emit()
            self.alert_message.emit(msg)
            return False

    def get_answer(self):
        """read data from socket"""
        response = self.socket.recv(config.MAX_POCKET_SIZE)
        # print(response)
        return response

    def get_protocol_from_response(self, response):
        """get protocol from server response"""
        try:
            protocol = JIM(request=response)
        except (UnicodeDecodeError, JSONDecodeError):
            response = decript_data(username=self.username, data=response)
            protocol = JIM(request=response)
        return protocol

    @log_exception(Exception)
    def send_request(self, request):
        """send request to server"""
        print("sending request")
        try:
            with LOCK:
                print("get request")
                # request = JIM().get_request(action, **kwargs)
                print(request)
                print("send request")
                self.socket.send(request)
                return self.proceed_answer()
        except Exception:
            print("error")

    def join_to_server(self):
        """try to join with server"""
        print("join")
        self.presence()
        request = JIM().get_request(
            action=JIM.JOIN,
            user=self.username,
            password=self.password)
        return self.send_request(request)

    def presence(self):
        """
        make presence with server
        get/send all necessary public keys
        for encrypting data
        """
        username_pk = get_public_key(username=self.username, to_str=True)
        if username_pk is None:
            generate_pair(username=self.username)
            username_pk = get_public_key(username=self.username, to_str=True)
        request = JIM().get_request(
            action=JIM.PRESENCE,
            user=self.username,
            public_key=username_pk)
        self.send_request(request)

    @Slot(str, str, str)
    def send_alert(self, msg, send_to, send_from):
        """send alert message to server"""
        print("send_alert")
        print(msg, send_to, send_from)
        request = JIM().get_request(
            action=JIM.ALERT,
            user=self.username,
            send_to=send_to,
            message=msg,
            send_from=send_from,
            info="delete_contact_history")
        print(request)
        self.send_request(request)

    def get_contacts(self):
        """send request to get user contacts"""
        print("get_contacts")
        request = JIM().get_request(
            action=JIM.CONTACTS,
            send_from=self.username,
            user=self.username)
        request = encript_server_data(request)
        self.send_request(request)

    def get_all_contacts(self):
        """send request to get all users on server"""
        print("get_all_contacts")
        request = JIM().get_request(action=JIM.CONTACTS, user=self.username)
        request = encript_server_data(request)
        contacts = self.send_request(request)
        return contacts

    def add_contact(self, username):
        """send request to add new contact to user list"""
        print("add_contact")
        request = JIM().get_request(
            action=JIM.ADD,
            send_from=self.username,
            message=username,
            user=self.username)
        request = encript_server_data(request)
        self.send_request(request)

    def del_contact(self, username):
        """send request to del contact from user list"""
        print("del_contacts")
        request = JIM().get_request(
            action=JIM.DELETE,
            send_from=self.username,
            message=username,
            user=self.username)
        self.send_request(request)

    def send_message(self, message, username, to):
        """send message to another user"""
        print("send_message")
        request = JIM().get_request(
            action=JIM.MESSAGE,
            send_from=username,
            send_to=to,
            message=message,
            user=self.username)
        request = encript_server_data(request)
        result = self.send_request(request)
        if result:
            self.database.save_msg(msg=message, user=username, to=to)

    def quit(self):
        """send request that client is closed or quit"""
        print("quit")
        request = JIM().get_request(action=JIM.QUIT, user=self.username)
        request = encript_server_data(request)
        self.send_request(request)

    def run(self) -> None:
        """start main transport event"""
        print("start transport")
        while self.is_active:
            time.sleep(1)
            with LOCK:
                try:
                    self.socket.settimeout(0.5)
                    self.proceed_answer()
                except OSError:
                    self.is_active = True
                except (ConnectionAbortedError, ConnectionError, ConnectionResetError):
                    self.is_active = False
                finally:
                    self.socket.settimeout(0.5)
        print("exit")



import dis
import types
from messenger2.common.decorators import log_exception


class BaseVerifier(type):
    """
    Base metaclass.
    Contains static methods.
    """
    def __init__(cls, name, bases, attr):
        super(BaseVerifier, cls).__init__(name, bases, attr)

    @staticmethod
    @log_exception(Exception)
    def start_verify(attr, prohibited_params, necessary_params):
        necessary_params_count = 0
        for key, value in attr.items():
            # print(value)
            try:
                closures = value.__closure__
                if closures is not None:
                    # print(closures)
                    for closure in closures:
                        content = closure.cell_contents
                        if isinstance(content, types.FunctionType):
                            necessary_params_count += \
                                BaseVerifier.check_instructions(content, prohibited_params=prohibited_params,
                                                                necessary_params=necessary_params)
                else:
                    if isinstance(value, types.FunctionType):
                        necessary_params_count += \
                            BaseVerifier.check_instructions(value, prohibited_params=prohibited_params,
                                                            necessary_params=necessary_params)
            except AttributeError:
                pass

        # print(necessary_params_count)
        if necessary_params_count < len(necessary_params):
            raise TypeError(
                f"Ошибка инициализации параметров {necessary_params}\n"
                f"Метод\\Атрибут(ы) отсутствуют")

    @staticmethod
    @log_exception(Exception)
    def check_instructions(
            content,
            prohibited_params: list = None,
            necessary_params: list = None):
        instr = dis.get_instructions(content)

        params = set()
        necessary_params_count = 0
        for i in instr:
            if i.opname == "LOAD_GLOBAL":
                params.add(i.argval)

            elif i.opname == "LOAD_ATTR":
                params.add(i.argval)

        if prohibited_params is not None:
            for param in params:
                if param in prohibited_params:
                    raise TypeError(
                        f"Ошибка инициализации параметра {param}\n"
                        f"Метод\\Атрибут запрещен для использования")

        if necessary_params is not None:
            for param in necessary_params:
                if param in params:
                    necessary_params_count += 1

        # print(params)

        return necessary_params_count


class ServerVerifier(BaseVerifier):
    """Server metaclass"""
    def __init__(cls, name, bases, attr):
        prohibited_params = ["connect"]
        necessary_params = ["AF_INET", "SOCK_STREAM"]
        cls.start_verify(attr, prohibited_params, necessary_params)

        super(ServerVerifier, cls).__init__(name, bases, attr)


class ClientVerifier(BaseVerifier):
    """Client metaclass"""
    def __init__(cls, name, bases, attr):
        prohibited_params = ["socket", "accept", "listen"]
        necessary_params = ["send_request", "get_answer"]
        cls.start_verify(attr, prohibited_params, necessary_params)

        super(ClientVerifier, cls).__init__(name, bases, attr)



import logging
from messenger2.logs.log_configs import client_log_config, server_log_config
import traceback
import sys

WAY = traceback.format_stack()[0].split()[1].strip('",').split('/')[-1]
if WAY == 'server_start.py':
    LOGGER = logging.getLogger(server_log_config.SERVER_LOGGER)
else:
    LOGGER = logging.getLogger(client_log_config.CLIENT_LOGGER)


def login_required():
    """
    decorator that check if user is register in system
    :return: func(*args, **kwargs) or alert request
    """
    def deco(func):
        def wrap(*args, **kwargs):
            if WAY == "server_start.py":
                from messenger2.server.core import Server
                from messenger2.protocols.JIM import JIM
                from socket import socket
                print("check login required")
                server = None
                client = None
                protocol = None
                for arg in args:
                    if isinstance(arg, Server):
                        server = arg

                    if isinstance(arg, socket):
                        client = arg

                    if isinstance(arg, JIM):
                        protocol = arg

                if server is None or client is None or protocol is None:
                    for key, item in kwargs.items():
                        if isinstance(item, Server):
                            server = item

                        if isinstance(item, socket):
                            client = item

                        if isinstance(item, JIM):
                            protocol = item

                print(server, protocol, client)
                if server is None:
                    raise Exception

                elif client is None:
                    raise Exception

                elif protocol is None:
                    msg = "Клиентский запрос использует неправильный протокол"
                    LOGGER.error(msg)
                    return JIM().get_request(action=JIM.ALERT, message=msg)
                else:

                    if protocol.presence_type or protocol.alert_type:
                        LOGGER.info("Подключение нового клиента")
                        return func(*args, **kwargs)
                    else:
                        database = server.db
                        active_users = [
                            user[1].login for user in database.get_active_user_list(
                                join_users=True)]
                        msg = "Попытка неавторизированного доступа"
                        if protocol.join_type:
                            LOGGER.info("Авторизация нового клиента")
                            return func(*args, **kwargs)
                        elif client in server.clients:
                            for login, client_socket in server.client_names.items():
                                if client_socket == client:
                                    if login in active_users:
                                        return func(*args, **kwargs)
                                    else:
                                        LOGGER.info(msg)
                                        return JIM().get_request(action=JIM.ALERT, message=msg)
                            LOGGER.info(msg)
                            return JIM().get_request(action=JIM.ALERT, message=msg)

                        else:
                            LOGGER.info(msg)
                            return JIM().get_request(action=JIM.ALERT, message=msg)

            else:
                LOGGER.critical("Несанкционированный запуск сервера")
                raise Exception
        return wrap

    return deco


def log_exception(exception):
    """
    decorator that log raised exceptions
    :param exception: Exception
    :return: func(*args, **kwargs)
    """
    def deco(func):
        def wrap(*args, **kwargs):

            try:
                log_msg = f'Была вызвана функция {func.__name__} с параметрами {args}, {kwargs} ' \
                          f'В модуле {func.__module__}'
                LOGGER.info(log_msg)
                return func(*args, **kwargs)

            except exception:

                log_msg = f'Возникла критическая ошибка в функции {func.__name__}' \
                          f'С параметрами {args}, {kwargs}' \
                          f'В модуле {func.__module__}' \
                          f'В файле {WAY}'
                LOGGER.critical(log_msg)
                LOGGER.critical(traceback.format_exc())
                sys.exit(-1)

        return wrap

    return deco


def thread_lock(locker):

    def deco(func):

        def wrap(*args, **kwargs):
            locker.acquire()
            result = func(*args, **kwargs)
            locker.release()
            return result

        return wrap

    return deco


class LogException:

    def __init__(self, exception):
        self.exception = exception

    def __call__(self, func):

        def deco(*args, **kwargs):
            try:
                log_msg = f'Была вызвана функция {func.__name__} с параметрами {args}, {kwargs} ' \
                          f'В модуле {func.__module__}'
                LOGGER.info(log_msg)
                return func(*args, **kwargs)

            except self.exception:
                log_msg = f'Возникла критическая ошибка в функции {func.__name__}' \
                          f'С параметрами {args}, {kwargs}' \
                          f'В модуле {func.__module__}' \
                          f'В файле {WAY}'
                LOGGER.critical(log_msg)
                LOGGER.critical(traceback.format_exc())
                sys.exit(-1)

        return deco


if __name__ == "__main__":
    pass



from PySide2.QtWidgets import QDialog
from PySide2.QtCore import QFile, Signal
from PySide2.QtUiTools import QUiLoader
import os
from messenger2 import config
from messenger2.client.gui.alert_window import AlertWindow


class AddWindow(QDialog):
    """
    Window for helping clients add new contacts
    """

    add_contact = Signal(str)

    def __init__(self, database, transport):
        super(AddWindow, self).__init__()
        self.database = database
        self.transport = transport
        self.alert = None
        self.setUI(os.path.join(config.CLIENT_UI_DIR, "add_client.ui"))
        self.update_list()

    def setUI(self, ui_file):
        """function that set up ui files"""
        ui = QFile(ui_file)
        ui.open(QFile.ReadOnly)
        loader = QUiLoader()
        self.ui = loader.load(ui)
        ui.close()

        self.ui.cancel_btn.clicked.connect(self.close)
        self.ui.choose_btn.clicked.connect(self.choose_contact)
        self.ui.update_btn.clicked.connect(self.update_list)

    def update_list(self):
        """function that update users list"""
        self.ui.selector.clear()
        users = self.transport.get_all_contacts()
        users.remove(self.transport.username)
        contacts = [contact.login for contact in self.database.get_contacts()]
        available_contacts = set(users) - set(contacts)

        if len(available_contacts) == 0:
            self.alert = AlertWindow(info_msg="Нет доступных контактов")
            self.alert.show()
        else:
            self.ui.selector.addItems(available_contacts)

    def choose_contact(self):
        """function that activates when client choose user to add"""
        contact = self.ui.selector.currentText()
        self.add_contact.emit(contact)
        self.update_list()

    def show(self) -> None:
        """show gui"""
        self.ui.show()

    def close(self) -> bool:
        """close window"""
        return self.ui.close()


from messenger2 import config
import traceback
from ipaddress import ip_address

WAY = traceback.format_stack()[0].split()[1].strip('",').split('/')[-1]
if WAY == 'server_start.py':
    LOGGER = "server_logger"
else:
    LOGGER = "client_logger"


class Port:
    """
    Descriptor for port checking
    """
    def __get__(self, instance, owner):
        return instance.__dict__[self.my_attr]

    def __set__(self, instance, value):

        # print("hello from port descriptor")
        # print(value)
        try:
            if value < 1024 or value > 65535:
                instance.__dict__[LOGGER].error('Неверно заданный порт')
                raise ValueError
        except ValueError:
            instance.__dict__[LOGGER].error(
                'В качастве порта может быть указано только число '
                'в диапазоне от 1024 до 65535.')
            instance.__dict__[LOGGER].error('Задан порт по умолчанию')

            instance.__dict__[self.my_attr] = config.SERVER_PORT
        else:
            instance.__dict__[self.my_attr] = value

        finally:
            print("port descriptor")
            print(f"real value {instance.__dict__[self.my_attr]}")

    def __delete__(self, instance):
        del instance.__dict__[self.my_attr]

    def __set_name__(self, owner, my_attr):
        # owner - владелец атрибута - <class '__main__.Worker'>
        # my_attr - имя атрибута владельца - hours, rate
        self.my_attr = my_attr


class Address:
    """
    Descriptor for ip check
    """

    def __get__(self, instance, owner):
        return instance.__dict__[self.my_attr]

    def __set__(self, instance, value):
        # print("hello from address descriptor")
        # print(value)
        try:
            if value != "":
                ip = ip_address(value)
            else:
                ip = ""
        except ValueError:
            instance.__dict__[LOGGER].error('адрес неправильной конфигурации')
            instance.__dict__[LOGGER].error('Задан адрес по умолчанию')

            instance.__dict__[self.my_attr] = config.USER_ADDRESS
        else:
            instance.__dict__[self.my_attr] = ip

        finally:
            print("address descriptor")
            print(f"real value: {instance.__dict__[self.my_attr]}")

    def __delete__(self, instance):
        del instance.__dict__[self.my_attr]

    def __set_name__(self, owner, my_attr):
        # owner - владелец атрибута - <class '__main__.Worker'>
        # my_attr - имя атрибута владельца - hours, rate
        self.my_attr = my_attr



from time import time
from messenger2.protocols.BaseProtocol import BaseProtocol
import json


class JIM(BaseProtocol):

    """
    JIM protocol for sending information
    """

    MAX_ACTION_LENGTH = 15
    MAX_USER_LENGTH = 25
    MAX_MESSAGE_LENGTH = 500
    NEEDED_KEYS = ['action', 'time']
    ACTION_CODES = [
        'authenticate',
        'quit',
        'presence',
        'check',
        'probe',
        'msg',
        'join',
        'leave',
        'add',
        'del',
        "get_contacts",
        "error"
    ]
    POSSIBLE_TYPES = [
        'action',
        'time',
        'message',
        'encoding',
        'send_from',
        'send_to',
        'user',
        "password",
        'response',
        'alert',
        "login",
        "public_key",
        "info"
    ]

    MESSAGE = "msg"
    PRESENCE = "presence"
    JOIN = "join"
    ADD = "add"
    DELETE = "del"
    CONTACTS = "get_contacts"
    ALERT = "error"
    QUIT = "quit"

    def __init__(self, request=None):
        self.request = None
        self.__response = {
            'action': 'response',
            'encoding': JIM.ENCODING,
            'time': time(),
            'response': 200,
            'alert': JIM.SERVER_CODES.get(200),
            'info': ""
        }
        if request is not None:
            self.request = json.loads(request.decode(encoding=self.ENCODING))

            if not JIM.__check_request(self.request):
                self.__response.update({
                    'response': 400,
                    'alert': JIM.SERVER_CODES.get(400)
                })
            else:
                self.__response.update({
                    'response': 200,
                    'alert': JIM.SERVER_CODES.get(200)
                })

    def get_request(self, action, **kwargs):
        """
        Generates request according to its action and kwargs
        :param action: JIM protocol action
        :param kwargs: possible data
        :return: encoded byte request
        """
        request = {
            'action': action,
            'encoding': JIM.ENCODING,
            'time': time(),
        }
        try:
            if kwargs is not None:
                request.update(kwargs)
            if JIM.__check_request(request):
                print("true")
                self.request = request
                return json.dumps(request).encode(JIM.ENCODING)
            else:
                raise ValueError
        except ValueError:
            print('Wrong request format or too long values')
            for key, item in kwargs.items():
                request.pop(key)
            self.request = request
            return json.dumps(request).encode(JIM.ENCODING)

    def get_response(self):
        """
        return encoded byte response
        :return: byte response
        """
        self.__response["user"] = self.get_user()
        print(self.__response)
        return json.dumps(self.__response).encode(JIM.ENCODING)

    def set_response_code(self, code):
        """
        Set response code
        :param code: code according to base protocol
        :return: None
        """
        self.__response.update({
            'response': code,
            'alert': JIM.SERVER_CODES.get(code)
        })

    def set_response_action(self, action):
        """
        Set response action
        :param action: new JIM action
        :return:
        """
        self.__response.update({
            'action': action
        })

    def set_info(self, info):
        """
        Set some additional info
        :param info: some string
        :return: None
        """
        self.__response.update({
            "info": info
        })

    def get_info(self):
        """
        Get info from request
        :return: string info
        """
        return self.request.get("info")

    def get_message_info(self):
        """
        Get message info
        :return: message, send_to and send_from
        """
        return self.request.get('message'), self.request.get(
            'send_to'), self.request.get('send_from')

    def get_user(self):
        """
        Get user from request
        :return: username
        """
        user = self.request.get('user')
        return user

    def set_user(self, user):
        """
        Set user in response
        :param user: username
        :return: None
        """
        self.__response.update(
            {"user": user}
        )

    def get_password(self):
        """
        get password from request
        :return: password
        """
        password = self.request.get("password")
        return password

    def get_public_key(self):
        """
        get public key from request
        :return: public key
        """
        public_key = self.request.get("public_key")
        return public_key

    def set_public_key(self, public_key):
        """
        set publick key in response
        :param public_key: string public key
        :return: None
        """
        self.__response.update({
            'public_key': public_key
        })

    def set_message(self, message, send_from=None):
        """
        set message in response
        :param message: message
        :param send_from: Optional
        :return: None
        """
        if send_from is None:
            self.__response.update({
                'message': message
            })
        else:
            self.__response.update({
                'message': message,
                "send_from": send_from
            })

    def set_info(self, info):
        """
        set info message in response
        :param info: some string
        :return: None
        """
        self.__response.update({
            'info': info
        })

    @staticmethod
    def __check_request(request):
        """
        check if request is valid
        :param request: JIN protocol request
        :return: True or False
        """
        for key in JIM.NEEDED_KEYS:
            if key == 'action':
                action_code = request.get(key)
                if action_code not in JIM.ACTION_CODES or len(
                        action_code) > JIM.MAX_ACTION_LENGTH:
                    return False
            if key not in request.keys():
                return False

        for key in request.keys():
            if key not in JIM.POSSIBLE_TYPES:
                return False

            if key == 'user':
                user = request.get(key)
                if user is None:
                    return False
                elif len(user) > JIM.MAX_USER_LENGTH:
                    return False

            if key == 'message':
                # print(request)
                if len(request.get(key)) > JIM.MAX_MESSAGE_LENGTH:
                    return False

        return True

    @staticmethod
    def decode_response(response):
        """
        Decode response from json
        :param response: response json
        :return:
        """
        return json.loads(response.decode(JIM.ENCODING))

    @property
    def message_type(self):
        return True if self.request.get('action') == 'msg' else False

    @property
    def join_type(self):
        return True if self.request.get('action') == 'join' else False

    @property
    def quit_type(self):
        return True if self.request.get('action') == 'quit' else False

    @property
    def presence_type(self):
        return True if self.request.get('action') == 'presence' else False

    @property
    def add_type(self):
        return True if self.request.get("action") == "add" else False

    @property
    def del_type(self):
        return True if self.request.get("action") == "del" else False

    @property
    def get_contacts_type(self):
        return True if self.request.get("action") == "get_contacts" else False

    @property
    def response_type(self):
        return True if self.request.get("action") == "response" else False

    @property
    def alert_type(self):
        return True if self.request.get("action") == "error" else False

    @property
    def response_alert_type(self):
        return True if self.__response.get("action") == "error" else False



from PySide2.QtWidgets import QDialog, QLabel, QVBoxLayout, QPushButton
from PySide2.QtCore import Qt


class AlertWindow(QDialog):
    """
    Window for showing some alert messages
    """

    def __init__(self, info_msg):
        super(AlertWindow, self).__init__()

        self.setWindowTitle("alert")

        Vbox = QVBoxLayout()
        msg = QLabel(text=info_msg)
        msg.setAlignment(Qt.AlignCenter)
        button = QPushButton("ОК")
        button.resize(100, 20)
        button.clicked.connect(self.close)
        Vbox.addWidget(msg)
        Vbox.addWidget(button)
        self.setLayout(Vbox)


from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_v1_5, AES
from messenger2.common.security.keys import get_server_private_key, get_private_key


def get_info(key, data):
    """
    decrypt data with private key
    :param key: rsa key
    :param data: encrypted data
    :return: decrypted data
    """
    enc_session_key, nonce, tag, enc_text = data[:key.size_in_bytes()], data[key.size_in_bytes(
    ):][:16], data[key.size_in_bytes():][16:][:16], data[key.size_in_bytes():][16:][16:]
    rsa = PKCS1_v1_5.new(key=key)
    session_key = rsa.decrypt(enc_session_key, sentinel=None)
    print(session_key)
    aes = AES.new(session_key, AES.MODE_EAX, nonce)
    text = aes.decrypt_and_verify(enc_text, tag)

    return text


def decript_data(username, data):
    """
    decrypt data on client side
    :param username: user login
    :param data: encrypted data
    :return: decrypted data
    """
    username_pk = get_private_key(username)
    return get_info(username_pk, data)


def decript_server_data(data):
    """
    decrypt data on server side
    :param data: encrypted data
    :return: decrypted data
    """
    server_pk = get_server_private_key()
    return get_info(server_pk, data)


if __name__ == "__main__":
    pass



import os
import sys
import configparser
from PySide2.QtCore import QCoreApplication

BASE_DIR = os.path.dirname(os.path.abspath(__file__))
SITE_PACKAGES_DIR = None
for path in sys.path:
    if path.endswith("site-packages"):
        SITE_PACKAGES_DIR = path
        break
        
QT_PLUGINS_PATH = os.path.join(
    SITE_PACKAGES_DIR, "PySide2\\plugins")
QCoreApplication.setLibraryPaths([QT_PLUGINS_PATH])

config = configparser.ConfigParser()
SERVER_INI = f"{BASE_DIR}\\server.ini"
config.read(SERVER_INI)
SETTINGS = config["SETTINGS"]

SERVER_PORT = SETTINGS.get("server_port")
LISTEN_ADDRESS = SETTINGS.get("listen_address")

DATABASE_PATH = SETTINGS.get("database_path")
DATABASE_NAME = SETTINGS.get("database_file")

USER_ADDRESS = "127.0.0.1"
USER_PORT = 7777

MAX_POCKET_SIZE = 1024
MAX_CONNECTIONS = 5

LOG_DIR = os.path.join(BASE_DIR, 'logs')
DATABASE_DIR = os.path.join(BASE_DIR, "databases")
CLIENT_DIR = os.path.join(BASE_DIR, "client")
SERVER_DIR = os.path.join(BASE_DIR, "server")
CLIENT_UI_DIR = "client\\gui\\ui"
SERVER_UI_DIR = "server\\gui\\ui"
DATABASE_ENGINE = f'sqlite:///{DATABASE_PATH}\\{DATABASE_NAME}'
CLIENT_DATABASE_ENGINE = 'sqlite:///client\\client_base_'
TEST_DATABASE_ENGINE = 'sqlite:///test_server_base.db3'
VENV_DIR = sys.executable

CLIENT_LOG_DIR = os.path.join(LOG_DIR, 'client_logs')
SERVER_LOG_DIR = os.path.join(LOG_DIR, 'server_logs')



class BaseProtocol:
    """
    Base protocol class
    """
    ENCODING = 'utf-8'
    SERVER_CODES = {
        100: 'базовое уведомление',
        101: 'важное уведомление',
        200: 'ОК',
        201: 'объект создан',
        202: 'подтверждение',
        400: 'неправильный запрос/JSON-объект',
        401: 'не авторизован',
        402: 'неправильный логин/пароль',
        403: 'пользователь заблокирован',
        404: 'пользователь/чат отсутствует на сервере',
        409: 'уже имеется подключение с указанным логином',
        410: 'адресат существует, но недоступен (offline)',
        500: 'ошибка сервера'
    }


import logging
from messenger2.config import SERVER_LOG_DIR
from logging.handlers import TimedRotatingFileHandler

"""
Creates main server config data
"""

FILE_LOG_NAME = 'server.log'
SERVER_LOGGER = 'messenger.server'

LOG = logging.getLogger(SERVER_LOGGER)

FILE_HANDLER = TimedRotatingFileHandler(
    f'{SERVER_LOG_DIR}\\{FILE_LOG_NAME}',
    when='D',
    interval=1,
    encoding='utf-8')
FILE_HANDLER.setLevel(logging.DEBUG)

FORMATTER = logging.Formatter("%(asctime)s - %(levelname)s : %(message)s")

FILE_HANDLER.setFormatter(FORMATTER)

LOG.addHandler(FILE_HANDLER)
LOG.setLevel(logging.DEBUG)

if __name__ == '__main__':
    print(SERVER_LOG_DIR)
    print(LOG)
    LOG.debug('test_error')



import logging
from messenger2.config import CLIENT_LOG_DIR

"""
Creates main client log config data
"""

FILE_LOG_NAME = 'client.log'
CLIENT_LOGGER = 'messenger.client'

LOG = logging.getLogger(CLIENT_LOGGER)

FILE_HANDLER = logging.FileHandler(
    f'{CLIENT_LOG_DIR}\\{FILE_LOG_NAME}',
    encoding='utf-8')
FILE_HANDLER.setLevel(logging.DEBUG)

FORMATTER = logging.Formatter("%(asctime)s - %(levelname)s : %(message)s")

FILE_HANDLER.setFormatter(FORMATTER)

LOG.addHandler(FILE_HANDLER)
LOG.setLevel(logging.DEBUG)

if __name__ == '__main__':
    print(CLIENT_LOG_DIR)
    print(LOG)
    LOG.debug('test_error')



from sqlalchemy import Column, Integer, String, DateTime, ForeignKey, create_engine, or_, Text
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base
import datetime
from messenger2.config import TEST_DATABASE_ENGINE, USER_PORT, USER_ADDRESS


class ServerDatabase:
    """
    Creates server database
    """
    Base = declarative_base()

    class AllUsers(Base):
        """
        all_users table
        """
        __tablename__ = "all_users"
        id = Column(Integer, primary_key=True)
        login = Column(String, default="", unique=True)
        password = Column(String)
        public_key = Column(Text)
        last_login = Column(DateTime, default=datetime.datetime.now())

        def __init__(self, login, password, public_key="user_public_key"):
            self.login = login
            self.password = password
            self.public_key = public_key
            self.last_login = datetime.datetime.now()

        def __repr__(self):
            return f"<AllUsers({self.login}, {self.password}, {self.public_key})>"

    class UserContacts(Base):
        """
        contacts table
        """
        __tablename__ = "contacts"
        id = Column(Integer, primary_key=True)
        user = Column(Integer, ForeignKey("all_users.id"))
        contact = Column(Integer, ForeignKey("all_users.id"))

        def __init__(self, user, contact):
            self.user = user
            self.contact = contact

        def __repr__(self):
            return f"<UserContacts({self.user}, {self.contact})>"

    class ActiveUsers(Base):
        """
        active_users table
        """
        __tablename__ = "active_users"
        id = Column(Integer, primary_key=True)
        user_id = Column(Integer, ForeignKey('all_users.id'))
        port = Column(Integer, default=122)
        ip_address = Column(String)
        log_time = Column(DateTime, default=datetime.datetime.now())

        def __init__(self, user_id, port, address):
            self.user_id = user_id
            self.port = port
            self.ip_address = address
            self.log_time = datetime.datetime.now()

        def __repr__(self):
            return f"<ActiveUsers({self.user_id}, {self.port}, {self.ip_address}, {self.log_time})>"

    class UsersHistory(Base):
        """users_history table"""
        __tablename__ = "users_history"
        id = Column(Integer, primary_key=True)
        user_id = Column(Integer, ForeignKey('all_users.id'))
        port = Column(Integer)
        ip_address = Column(String)
        log_time = Column(DateTime, default=datetime.datetime.now())

        def __init__(self, user_id, port=USER_PORT, address=USER_ADDRESS):
            self.user_id = user_id
            self.port = port
            self.ip_address = address
            self.log_time = datetime.datetime.now()

        def __repr__(self):
            return f"<UsersHistory({self.user_id}, {self.port}, {self.ip_address}, {self.log_time})>"

    def __init__(self, engine):
        self._engine = create_engine(
            engine, connect_args={
                'check_same_thread': False})
        self.Base.metadata.create_all(self._engine)
        self._session = sessionmaker(bind=self._engine)

    @property
    def session(self):
        """
        creates new session
        :return: session
        """
        return self._session()

    def save(self, info: list):
        """
        save list of data
        :param info: array of Server database objects
        :return: None
        """
        session = self.session
        for data in info:
            session.add(data)

        session.commit()
        session.close()

    def update_user_history(self, username, ip_address, port):
        """
        update history
        :param username: login
        :param ip_address: current ip
        :param port: current port
        :return: None
        """
        session = self.session
        user = session.query(self.AllUsers).filter_by(login=username)
        if user.count() != 0:
            history = session.query(
                self.UsersHistory).filter_by(
                user_id=user.first().id)
            if history.count() == 0:
                user_history = self.UsersHistory(
                    user_id=user.first().id, address=ip_address, port=port)
                session.add(user_history)
            else:
                history.update({self.UsersHistory.log_time: datetime.datetime.now(
                ), self.UsersHistory.ip_address: ip_address, self.UsersHistory.port: port})
            session.commit()
        session.close()

    def get_user(self, login):
        """
        Get user from database
        :param login: login
        :return: None
        """
        session = self.session
        user = session.query(self.AllUsers).filter_by(login=login).first()
        session.close()
        return user

    def save_user_pk(self, username, public_key):
        """
        save user pk in database
        :param username: login
        :param public_key: string public key
        :return: None
        """
        session = self.session
        session.query(self.AllUsers).filter_by(login=username).update(
            {self.AllUsers.public_key: public_key})
        session.commit()
        session.close()

    def get_user_pk(self, username):
        """
        get user public key from database
        :param username: login
        :return: None
        """
        session = self.session
        user = session.query(self.AllUsers).filter_by(login=username).first()
        public_key = user.public_key
        session.close()
        return public_key

    def register_user(self, login, password):
        """
        Register new user in database
        :param login: login
        :param password: hash password
        :return: None
        """
        session = self.session
        user = self.AllUsers(login=login, password=password)
        session.add(user)
        user = session.query(self.AllUsers).filter_by(login=login).first()
        user_history = self.UsersHistory(user_id=user.id)
        session.add(user_history)
        session.commit()
        session.close()

    def delete_active_user(self, login):
        """
        Delete user from active user list
        :param login: login
        :return: None
        """
        session = self.session
        user_id = session.query(
            self.AllUsers).filter_by(
            login=login).first().id
        session.query(self.ActiveUsers).filter_by(user_id=user_id).delete()
        session.commit()
        session.close()

    def clear_active_users(self):
        """
        Clear current active list
        :return: None
        """
        session = self.session
        session.query(self.ActiveUsers).delete()
        session.commit()
        session.close()

    def delete(self, query):
        """
        delete query
        :param query: database query
        :return:
        """
        session = self.session
        query.delete()
        session.commit()
        session.close()

    def get_user_list(self):
        """
        get all users from database
        :return: users
        """
        session = self.session
        users = session.query(self.AllUsers).all()
        session.close()
        return users

    def get_contacts(self, username):
        """
        get all contacts from database
        :param username: login (not necessary)
        :return: array of users
        """
        session = self.session
        contacts = session.query(self.AllUsers.login).all()
        session.close()
        return [contact[0] for contact in contacts]

    def get_user_contacts(self, username):
        """
        Get user contacts from database
        :param username: login
        :return: user contacts
        """
        session = self.session
        user = session.query(
            self.AllUsers.id).filter_by(
            login=username).first()

        user_contacts = session.query(
            self.UserContacts,
            self.AllUsers).filter_by(
            user=user.id) .join(
            self.AllUsers,
            self.UserContacts.contact == self.AllUsers.id).all()

        return [user[1].login for user in user_contacts]

    def get_history_list(self, join_users=False):
        """
        get history list
        :param join_users: bool to join with all_users or not
        :return: history array
        """
        session = self.session
        if join_users:
            history = session.query(self.UsersHistory, self.AllUsers.login).join(
                self.AllUsers, self.AllUsers.id == self.UsersHistory.user_id).all()
        else:
            history = session.query(self.UsersHistory).all()
        session.close()
        return history

    def get_active_user_list(self, join_users=False):
        """
        get active user list from database
        :param join_users: bool to join with all_users or not
        :return: active_users array
        """
        session = self.session
        if join_users:
            active_users = session.query(self.ActiveUsers, self.AllUsers).join(
                self.AllUsers, self.AllUsers.id == self.ActiveUsers.user_id).all()
        else:
            active_users = session.query(self.ActiveUsers).all()
        session.close()
        return active_users

    def check_active_user(self, username):
        """
        check if user is in current active users list
        :param username: login
        :return: True or False
        """
        user = self.get_user(username)
        session = self.session
        active_user = session.query(
            self.ActiveUsers).filter_by(
            user_id=user.id).count()
        session.close()
        if active_user != 0:
            return True
        return False

    def clear_db(self):
        """
        Clear all database
        :return: None
        """
        session = self.session
        session.query(self.AllUsers).delete()
        session.query(self.UsersHistory).delete()
        session.query(self.ActiveUsers).delete()
        session.query(self.UserContacts).delete()
        session.commit()
        session.close()

    def add_contact(self, username, login):
        """
        Add contact to username contact list
        :param username: user login
        :param login: contact login
        :return: True or False
        """
        session = self.session
        user = session.query(self.AllUsers).filter_by(login=username)
        contact = session.query(self.AllUsers).filter_by(login=login)
        if user.count() != 0 and contact.count() != 0:
            if session.query(
                    self.UserContacts).filter_by(
                    user=user.first().id,
                    contact=contact.first().id).count() == 0:
                # add contact
                contact = self.UserContacts(
                    user=user.first().id, contact=contact.first().id)
                session.add(contact)
                session.commit()
                session.close()
                return True
        session.close()
        return False

    def del_contact(self, username, login):
        """
        Delete contact from user list
        :param username: user login
        :param login: contact login
        :return: True or False
        """
        session = self.session
        user = session.query(self.AllUsers).filter_by(login=username)
        contact = session.query(self.AllUsers).filter_by(login=login)
        print(user.first())
        print(user.first())
        if user.count() != 0 and contact.count() != 0:
            contact_to_del = session.query(
                self.UserContacts).filter_by(
                user=user.first().id,
                contact=contact.first().id)
            if contact_to_del.count() != 0:
                # delete contact
                contact_to_del.delete()
                session.commit()
                session.close()
                return True
        session.close()
        return False

    def check_user(self, login):
        """
        Check if user is register in database
        :param login: login
        :return: True or False
        """
        session = self.session
        user = session.query(self.AllUsers).filter_by(login=login)
        session.close()
        return True if user.count() != 0 else False

    def check_user_password(self, login, password):
        """
        Check user password
        :param login: login
        :param password: user password to check
        :return: True or False
        """
        session = self.session
        user = session.query(self.AllUsers).filter_by(login=login).first()
        session.close()
        if password == user.password:
            return True
        else:
            return False

    def add_user(self, login, password):
        """
        Add new user to database
        :param login: login
        :param password: user password
        :return:
        """
        session = self.session
        user = self.AllUsers(login=login, password=password)
        session.add(user)
        session.commit()
        session.close()


class ClientDatabase:
    """
    Create client database
    """
    Base = declarative_base()

    class Contacts(Base):
        """
        contacts table
        """
        __tablename__ = "contacts"
        id = Column(Integer, primary_key=True)
        login = Column(String)

        def __init__(self, login):
            self.login = login

        def __repr__(self):
            return f"<Contacts({self.login})"

    class HistoryMessage(Base):
        """
        history message table
        """
        __tablename__ = "history"
        id = Column(Integer, primary_key=True)
        user = Column(String)
        to = Column(String)
        msg = Column(String)
        when = Column(DateTime, default=datetime.datetime.now())

        def __init__(self, msg, user, to):
            self.msg = msg
            self.user = user
            self.to = to
            self.when = datetime.datetime.now()

        def __repr__(self):
            return f"<History({self.user}, {self.to}, {self.msg}, {self.when})>"

    @property
    def session(self):
        """
        Creates new session
        :return: session
        """
        return self._session()

    def __init__(self, engine):
        self._engine = create_engine(
            engine, connect_args={
                'check_same_thread': False})
        self.Base.metadata.create_all(self._engine)
        self._session = sessionmaker(bind=self._engine)

    def save_msg(self, user, to, msg):
        """
        save contacts message
        :param user: sender
        :param to: receiver
        :param msg: message
        :return: None
        """
        log = self.HistoryMessage(msg=msg, user=user, to=to)
        session = self.session
        session.add(log)
        session.commit()
        session.close()

    def check_contact(self, login):
        """
        check if contact is in user contact list
        :param login: contact login
        :return: True or False
        """
        session = self.session
        client = session.query(self.Contacts).filter_by(login=login)
        session.close()
        if client.count() == 0:
            return False
        return True

    def add_client(self, login):
        """
        Add new contact to user contact list
        :param login: contact login
        :return: None
        """
        session = self.session
        client = self.Contacts(login=login)
        session.add(client)
        session.commit()
        session.close()

    def del_client(self, login):
        """
        Delete contact from user contact list
        :param login: contact login
        :return: None
        """
        session = self.session
        client = session.query(self.Contacts).filter_by(login=login)
        client.delete()
        session.commit()
        session.close()

    def add_clients(self, logins: list):
        """
        Add multiple client to user contact list
        :param logins: array of users login
        :return: None
        """
        session = self.session
        session.query(self.Contacts).delete()
        for login in logins:
            client = self.Contacts(login=login)
            session.add(client)
        session.commit()
        session.close()

    def get_contacts(self):
        """
        Get user contact list
        :return: contacts
        """
        session = self.session
        contacts = session.query(self.Contacts).all()
        session.close()
        return contacts

    def get_contact_history(self, login):
        """
        get current contact history
        :param login: contact login
        :return: contact history
        """
        session = self.session
        history = session.query(
            self.HistoryMessage).filter(
            or_(
                self.HistoryMessage.user == login,
                self.HistoryMessage.to == login)).order_by(
                self.HistoryMessage.when).all()
        session.close()
        return history

    def delete_contact_history(self, login):
        """
        Delete current contact history
        :param login: contact login
        :return: None
        """
        session = self.session
        session.query(self.HistoryMessage).filter(
            or_(self.HistoryMessage.user == login, self.HistoryMessage.to == login)).delete()
        session.commit()
        session.close()

    def clear_db(self):
        """
        Clear current database
        :return: None
        """
        session = self.session
        session.query(self.Contacts).delete()
        session.query(self.HistoryMessage).delete()
        session.commit()
        session.close()


if __name__ == "__main__":
    info = []
    db = ServerDatabase(engine=TEST_DATABASE_ENGINE)
    session = db.session
    test_query = session.query(db.AllUsers)
    db.delete(test_query)
    user = db.AllUsers(
        name="test",
        surname="test",
        login="test_1",
        password='1111')
    find_query = db.session.query(db.AllUsers).filter_by(login="test_1")
    if find_query.count() == 0:
        info.append(user)
        db.save(info)
        info = []
        find_query = db.session.query(db.AllUsers).filter_by(login="test_1")
    active_user = db.ActiveUsers(
        user_id=find_query.first().id,
        port=100,
        address="localhost")
    info.append(active_user)
    db.save(info)
    users = db.get_user_list()
    print(users)
    print(users[0].login)
    db.clear_db()



from setuptools import setup, find_packages
setup(name="python_messenger_client",
      version="0.0.5",
      description="Messenger Client",
      author="Sergey Kurskiy",
      author_email="kurskiisergey@mail.ru",
      packages=find_packages(),
      install_requires=['PySide2', 'sqlalchemy', 'pycryptodome', 'pycryptodomex'],
      package_data={
            '': ['*.ini', "client\\gui\\ui\\*.ui"],
      },
      )



from PySide2.QtWidgets import QDialog
from PySide2.QtCore import QFile, Slot
from PySide2.QtUiTools import QUiLoader
from messenger2 import config
import os
from messenger2.client.gui.alert_window import AlertWindow
from messenger2.client.gui.client_window import ClientWindow
from messenger2.client.transport import ClientTransport
from messenger2.common.security.hash_password import get_hash_from_password


class WelcomeWindow(QDialog):

    """
    Welcome window. For entering on server
    if such user is exists
    """

    def __init__(self, ip_address, port, username=None, password=None):
        super(WelcomeWindow, self).__init__()
        self.database = None
        self.transport = None
        self.ip_address = ip_address
        self.port = port
        self.alert = None
        self.ui = None
        self.username = username
        self.password = password
        self.setUI(os.path.join(config.CLIENT_UI_DIR, "welcome.ui"))

    def setUI(self, ui_file):
        """function that set up ui files"""
        ui = QFile(ui_file)
        ui.open(QFile.ReadOnly)
        loader = QUiLoader()
        self.ui = loader.load(ui)
        ui.close()

        self.ui.cancel_btn.clicked.connect(self.close)
        self.ui.enter_btn.clicked.connect(self.register_user)

    def register_user(self):
        """function that check input login and password for auth"""
        username = self.ui.user_edit.text()
        password = get_hash_from_password(
            password=self.ui.pwd_edit.text(), salt=username)

        self.transport = ClientTransport(
            ip_address=self.ip_address,
            port=self.port,
            username=username,
            password=password)
        self.transport.alert_message.connect(self.show_alert)
        self.transport.connect_to_server()
        if self.transport.is_active:
            self.database = self.transport.database
            self.transport.setDaemon(True)
            self.transport.start()
            window = ClientWindow(
                database=self.database,
                transport=self.transport,
                username=username,
                ui_file=os.path.join(
                    config.CLIENT_UI_DIR,
                    "client.ui"))
            window.show()
            self.close()
        else:
            self.transport = None
            pass
            # self.alert = AlertWindow(info_msg="Неверный логин или пароль")
            # self.alert.show()

    @Slot(str)
    def show_alert(self, msg):
        """function that shows alert messages"""
        self.alert = AlertWindow(info_msg=msg)
        self.alert.show()

    def close(self) -> bool:
        """close window"""
        return self.ui.close()

    def show(self) -> None:
        """show gui"""
        if self.username is not None and self.password is not None:
            self.ui.show()
            self.ui.user_edit.setText(self.username)
            self.ui.pwd_edit.setText(self.password)
            self.register_user()
        else:
            self.ui.show()



from Crypto.PublicKey import RSA
import os
from messenger2.protocols.JIM import JIM
from messenger2 import config

SERVER_KEYS_DIR = os.path.join(config.SERVER_DIR, "server_keys")
CLIENT_KEYS_DIR = os.path.join(config.CLIENT_DIR, "client_keys")


def generate_pair(username, is_client=True):
    """
    generate rsa keys if they are not created
    :param username: login
    :param is_client: bool value if we check on server or client side
    :return: None
    """
    dir = CLIENT_KEYS_DIR
    if not is_client:
        dir = SERVER_KEYS_DIR
        username = "server"
    try:
        files = os.listdir(dir)
    except FileNotFoundError:
        os.mkdir(dir)
        files = os.listdir(dir)
    if f"public_{username}.pem" and f"private_{username}.pem" in files:
        print("keys are already exists")
    else:
        key = RSA.generate(2048)
        with open(os.path.join(dir, f"private_{username}.pem"), "wb") as f:
            f.write(key.export_key())

        with open(os.path.join(dir, f"public_{username}.pem"), "wb") as f:
            f.write(key.public_key().export_key())


def get_public_key(username, to_str=False):
    """
    get public key from client side
    :param username: login
    :param to_str: bool to get str instead of rsa object
    :return: rsa object, string or None if there is no key
    """
    try:
        key = RSA.import_key(
            open(
                os.path.join(
                    CLIENT_KEYS_DIR,
                    f"public_{username}.pem")).read())
        if to_str:
            key = str(key.public_key().export_key(), encoding=JIM.ENCODING)
        return key
    except (FileNotFoundError, FileExistsError):
        return None


def get_private_key(username, to_str=False):
    """
    get private key from client side
    :param username: login
    :param to_str: bool to get str instead of rsa object
    :return: rsa object, string or None if there is no key
    """
    try:
        key = RSA.import_key(
            open(
                os.path.join(
                    CLIENT_KEYS_DIR,
                    f"private_{username}.pem")).read())
        if to_str:
            key = str(key.public_key().export_key(), encoding=JIM.ENCODING)
        return key
    except (FileNotFoundError, FileExistsError):
        return None


def get_server_public_key(to_str=False):
    """
    get public key from server
    :param to_str: bool to get str instead of rsa object
    :return: rsa object, string or None if there is no key
    """
    try:
        with open(os.path.join(SERVER_KEYS_DIR, "public_server.pem"), "rb") as server_key:
            key = RSA.import_key(server_key.read())
            if to_str:
                key = str(key.public_key().export_key(), encoding=JIM.ENCODING)
            return key
    except (FileNotFoundError, FileExistsError):
        return None


def get_client_server_public_key(to_str=False):
    """
    get public server key from client side
    :param to_str: bool to get str instead of rsa object
    :return: rsa object, string or None if there is no key
    """
    try:
        with open(os.path.join(CLIENT_KEYS_DIR, "public_server.pem"), "rb") as server_key:
            key = RSA.import_key(server_key.read())
            if to_str:
                key = str(key.public_key().export_key(), encoding=JIM.ENCODING)
            return key
    except (FileNotFoundError, FileExistsError):
        return None


def get_server_private_key(to_str=False):
    """
    get private server key from server side
    :param to_str: bool to get str instead of rsa object
    :return: rsa object, string or None if there is no key
    """
    try:
        with open(os.path.join(SERVER_KEYS_DIR, "private_server.pem"), "rb") as server_key:
            key = RSA.import_key(server_key.read())
            if to_str:
                key = str(key.export_key(), encoding=JIM.ENCODING)
            return key
    except (FileNotFoundError, FileExistsError):
        return None


def save_server_public_key(server_pk):
    """
    save public server key onn client side
    :param server_pk: public rsa server key string
    :return: None
    """
    with open(os.path.join(CLIENT_KEYS_DIR, "public_server.pem"), "wb") as server_key:
        server_key.write(bytes(server_pk, encoding=JIM.ENCODING))


if __name__ == "__main__":
    generate_pair("server")
